/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/**
 * this function builds global data structures used by the Autozyg programs.
 * 
 * proc_auto() has as its input the results of parsing the parameter statements
 * for the Autozyg programs, lm_auto, lm_lods, lm_bayes, m_markers, lm_pvals.
 * proc_auto() uses many of the proc_drop() data structures and functions.
 * (Think of proc_auto as an extension of proc_drop.)
 * This function checks parameter statement data for consistency and complete-
 * ness and, if there are no errors, builds the global data structures.
 *
 * One chromosome only is selected
 * There must be at least one marker
 * For lm_pval, lm_map and lm_ibdtests no trait is selected, although
 * there is trait data for lm_pval.
 * For lm_auto, the trait is linked or unlinked.
 * For lm_lods, lm_markers, lm_bayes and lm_schnell, the trait has varying 
 * test positions along the chromosome.
 *
 * Any combination of sex-not-specified maps or sex-specific maps may be used
 *
 *                                     MAJ  revs 5/2000, 1/2002, 8/2003, 6/2005 
 */

int proc_auto (void)
{
   int   lnkd, entry, entry2, sex, m;
   int   sxsp_map = FALSE;
   double d;
   SIM_PTR p_s;
   DIST_PTR p_d;
   UNLNKD_PTR p_u;
   TLIST_PTR p_t;
   VMAP_PTR p_v;
   PRNTMAP_PTR p_m;
   PRNTVMAP_PTR p_vm;
   PRNTLOC_PTR p_l;
   PSMAP_PTR p_si;

   if (!test_atz_sel ())                /* checks for select statement        */
      if (proc_err ())
         return FALSE;
   if (n_err)
      return proc_end ();

   entry = FALSE;                       /* selects one chromosome only        */
   p_s = nxt_sim_chrom (&entry);

   if (!test_mark_map (p_s, &sxsp_map)) /* checks marker map & trait map      */
      if (proc_err ())                  /* (sex-averaged & sex-specific)      */
         return FALSE;
   if (IBDDROP || LM_AUTO)
      if (!test_trait_map (p_s, &sxsp_map))
         if (proc_err ())
            return FALSE;
   if (LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
      if (!test_trait_vmap (p_s, &sxsp_map))
         if (proc_err ())
            return FALSE;
   if (n_err)                           /* proc_err() quits with "F" errors;  */
      return proc_end ();               /* n_err counts "E" errors            */

   lnkd = TRUE;                         /* adds mapped trait to trait list    */
   entry = FALSE;                       /* (a, i)                             */
   if (p_d = nxt_mapd_trait (&entry, FEM, p_s))
      if (!test_trait_list (lnkd, p_d->no))
         if (proc_err ())
            return FALSE;

   lnkd = FALSE;                        /* adds unlinked trait to trait list  */
   entry = FALSE;                       /* (a, i)                             */
   if (p_u = nxt_unlnkd_trait (&entry))
      if (!test_trait_list (lnkd, p_u->no))
         if (proc_err ())
            return FALSE;

   lnkd = FALSE;                        /* adds trait with varying (test)     */
   if (p_v = nxt_trait_vmap (FEM, p_s)) /* positions to trait list (l, y, e)  */
      if (!test_trait_list (lnkd, p_v->no))
         if (proc_err ())
            return FALSE;

   if (!test_trait_typ ())              /* establishes trait type            */
      if (proc_err ())
         return FALSE;

   entry = FALSE;                       /* for ea marker, check allele freqs  */
   if (!IBDDROP)
      while (m = nxt_mark_no (&entry, p_s))
         if (!test_mark_loci (p_s, m))
            if (proc_err ())
               return FALSE;

   entry = FALSE;                       /* for the trait, check allele freqs  */

   if (p_t = nxt_trait (&entry))
      if (!IBDDROP && !(MARKERDROP && PedDatTyp == INHERITANCE))
      {
         if (!test_trait_loci (p_t->no))
            if (proc_err ())
               return FALSE;
         if ((LM_MARKERS && TraitTyp == QUANTITATIVE)
             || LM_SCHNELL || LM_TWOQTL)
         {
            if (!test_trait_vars (p_t->no))
               if (proc_err ())
                  return FALSE;
            if (!test_trait_genotyp (p_t->no))
               if (proc_err ())
                  return FALSE;
         }
      }

   if (n_err)
      return proc_end ();

   for (sex = FEM; sex <= AVG; ++sex)   /* makes the "printmap"
                                         * assigns loci to sex-specific maps.
                                         * for maps not present,     
                                         * defaults to avg, then F or M maps  */
   {
      p_m = add_prntmap (p_s, sex);     /* puts markers into map              */

      entry = FALSE;
      entry2 = FALSE;

      if (p_s->no_mark_map == 0)
      {
         p_l = add_only_mark_locus (p_m);
         sel_mark_locus (p_s, p_l, &entry2);
      }
      else
         while (p_d = nxt_mapd_mark (&entry, sex, p_s))
         {
            p_l = add_mark_locus (p_m, p_d);
            sel_mark_locus (p_s, p_l, &entry2);
         }

      entry = FALSE;                    /* adds the linked trait, if present  */
      if (p_d = nxt_mapd_trait (&entry, sex, p_s))
      {
         p_l = add_trait_locus (p_m, p_d);
         sel_trait_locus (p_l);
      }

      if (LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
      {                                 /* makes the map for trial trait pos  */
         p_v = nxt_trait_vmap (sex, p_s);
         p_vm = add_vprntmap (p_v, sex); /* adds trait positions             */
         add_trait_vext (p_vm, sex);    /* for start & end of chrom           */
         add_trait_vint (p_vm, p_m, sex); /* and for marker intvs            */
         sel_trait_pos (p_vm, p_m);     /* now condenses trait maps for sel   */
      }

      shrink_prntmap (p_m);             /* removes non-selected markers       */

      entry = FALSE;                    /* finds distances between adj loci   */
      while (p_l = nxt_prntmap_locus (&entry, p_m))
         fix_locus_dist (p_m, p_l);

      if ((sxsp_map && (sex == FEM || sex == MAL)) || (!sxsp_map && sex == AVG))
      {
         prnt_map (p_m, sex);           /* prints the marker & lnkd trait map */

         if (LM_MAP)                    /* checks distances between markers   */
         {                              /* not less than specified minimum    */
            entry = FALSE;
            d = rf2dist (RF_bound);
            while (p_l = nxt_prntmap_locus (&entry, p_m))
               if (!test_loc_mindist (p_l, d))
                  if (proc_err ())
                     return FALSE;
         }

         entry = FALSE;                 /* checks trait distances (a, i)      */
         while (p_l = nxt_prntmap_locus (&entry, p_m))
            if (!test_trait_dist (p_m, p_l))
               if (proc_err ())
                  return FALSE;

         if (LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
            if ((sxsp_map && (sex == FEM || sex == MAL)) || (!sxsp_map && sex == AVG)) /* prints trial trait pos      */
               prnt_vmap (first_vpmap[sex], p_m->no_mark_sel, sex);
      }
   }
   if (LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
   {
      if (!test_trait_vext (p_s))       /* tests for the same no. of F & M    */
         if (proc_err ())               /* pos at start and end of chrom      */
            return FALSE;
      if (!test_trait_vint (p_s))       /* and in ea interval                 */
         if (proc_err ())
            return FALSE;
   }
   if (n_err)
      return proc_end ();

   if (!IBDDROP)
   {
      prnt_mark_freqs (p_s);            /* prints marker allele freqs         */

      if (!(MARKERDROP && PedDatTyp == INHERITANCE)) /* FIXME: should'nt need */
      {
         prnt_trait_freqs (p_m);        /* prints trait allele freqs          */

         prnt_unlnkd_freqs ();
      }
   }
   prnt_vtrait_freqs (p_s);

   fprintf (stdout, "\n");

   if (n_err)
      return proc_end ();

   NMarks = p_m->no_mark_sel;           /* counts loci                        */
   NLnkd = p_m->no_traits;
   if (p_u)
      ++NUnlnkd;
   if (p_v)
      ++NVary;
   NTraits = NLnkd + NUnlnkd + NVary;

   NLoci = NMarks + 1;                  /* right now we count the trait, 
                                         * whether or not one is used         */

   mak_traitnam (p_t);                  /* and trait name                     */

   alloc_marek ();                      /* stores marker recomb fracts in
                                         * Marek[][]                          */
   mak_marek ();

   mak_tarek ();                        /* stores trait recomb fract and pos  */

   if (NVary)                           /* sets NTPos, no. trait positions    */
   {
      NTPos = 0;
      p_si = first_vpmap[FEM]->p_sintv;
      while (p_si)
      {
         NTPos += p_si->no_pos;
         p_si = p_si->p_next;
      }
      NTPos1 = NTPos + 1;

      mak_locpos ();                    /* stores info about all loci         */

      alloc_intval ();                  /* sets positions in IntVal[]         */
      mak_intval ();                    /* & recomb fracts in RecVal[][]      */
      mak_recval ();                    /* (arrays are 2 greater than NTPos   */
   }

   entry = FALSE;                       /* for each locus, stores allele      */
   p_m = nxt_prntmap (&entry, FEM);     /* frequencies in Qq[][]              */
   entry = FALSE;
   if (!IBDDROP)
   {
      while (p_l = nxt_prntmap_locus (&entry, p_m))
         if (!(p_l->locus_typ == TD && PedDatTyp == INHERITANCE))
            mak_allefrq (p_m, p_l, p_l->no);
      if (p_u)
         mak_allefrq (NULL, NULL, p_u->no);
      else if (p_v)
         mak_allefrq (NULL, NULL, p_v->no);
      else if (!NTraits)
         mak_allefrq (NULL, NULL, 0);
   }

   if (!test_marknam (p_s))             /* checks marker names                */
      if (proc_err ())
         return FALSE;

   if (MARKERDROP
       || LM_MAP || LM_PVAL || LM_IBDTESTS || LM_AUTO
       || LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
      if (!test_markdat (p_s))          /* checks marker data                */
         if (proc_err ())
            return FALSE;

   if (!n_err)
   {
      if (mak_marknam (p_s))            /* stores and prints marker names    */
         prnt_marknam (p_s);            /* prints names, if input            */
   }

   if (!n_err)
   {
      mak_markdat (p_s);                /* stores and prints marker data     */
      prnt_markdat (p_s);
   }

   if (IBDDROP || LM_AUTO)
   {                                    /* checks proband gametes            */
      if (!test_probgam (p_s))          /* and locus window                  */
         if (proc_err ())
            return FALSE;
      if (LM_AUTO)                      /* checks pattern labels             */
         if (!test_patlab (p_s))
            if (proc_err ())
               return FALSE;
      if (!n_err)
      {
         mak_probgam ();                /* stores proband gametes            */
         if (LM_AUTO)
            mak_patlab ();              /* stores window pattern labels      */
         prnt_probgam ();               /* prints locus window, proband      */
      }                                 /* gametes and window patterns       */
   }

   if (LM_BAYES)                        /* checks pseudo-priors              */
   {
      if (!test_ps_pr ())
         if (proc_err ())
            return FALSE;
   }

   if (!test_atz_pars ())               /* checks MC computational parametrs */
      if (proc_err ())
         return FALSE;

   if (!test_ped_trtdat ())             /* checks input record parmeters     */
      if (proc_err ())
         return FALSE;

   if (LM_PVAL || LM_IBDTESTS)
   {                                    /* checks affected individuals       */
      if (!test_affind ())
         if (proc_err ())
            return FALSE;
      mak_affind ();                    /* stores affected individuals        */
      prnt_affind ();                   /* (if known) and prints              */
   }

   if (!test_penetrnc ())
      if (proc_err ())
         return FALSE;

   if (n_err)
      return proc_end ();

   if (NTraits && (TraitTyp == QUANTITATIVE)) /* stores geno means & variance */
   {
      alloc_trait ();
      mak_trait (p_t->no, 0, 0);
      prnt_qtrait_pars (p_s);           /* prints means and variances         */
   }

   mak_ped_dat ();                      /* set up stuff for reading           */

   prnt_ped_dat ();                     /* reports pedigree data to be read   */

   prnt_trt_typ ();                     /* prints trait type                  */

   prnt_penetrnc ();                    /* prints incomplete penetrances for
                                         * discrete trait data                */

   prnt_atz_pars ();                    /* prints MC parameters               */

   return TRUE;
}

/* -------- test fncs:  test results of processing input statements --------  */

/* ----------------------- test functions for autozyg ----------------------- */

/** 
 * gets the requested trait map.
 * returns the pointer to the trait map for the selected chromosome and 
 * gender, using gender defaults
 */
VMAP_PTR nxt_trait_vmap (int sx, SIM_PTR p_sel)
{
   int   i, j;
   VMAP_PTR p, p_vmap = NULL;

   for (i = 1; !p_vmap && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map    */
         j = sx;                        /* present in this order             */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;

      if (p = first_trait_vmap_node[j])
      {
         while (p->p_next && (p->chrom_no < p_sel->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_sel->chrom_no)
            p_vmap = p;
      }
   }
   return p_vmap;
}

/**
 * gets the requested beginning and/or ending map for trial trait positions.
 * returns the pointer to the trait map for the beginning or end of the
 * chromosome for the specified gender, using gender defaults.
 */
XTMAP_PTR nxt_trait_vext (int sx, PRNTVMAP_PTR p_pr, int d)
{
   int   i, j;
   VMAP_PTR p;
   XTMAP_PTR p_x = NULL;

   for (i = 1; !p_x && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map     */
         j = sx;                        /* data present in this order         */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;
      if (p = first_trait_vmap_node[j])
      {
         while (p->p_next && (p->chrom_no < p_pr->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_pr->chrom_no)
            if (d == BDM)
               p_x = p->p_beg;
            else if (d == EDM)
               p_x = p->p_end;
      }
   }
   return p_x;
}

/**
 * gets the requested interval proportions map.
 * returns the pointer to the trait interval proportions list for the
 * specified interval and gender using gender defaults              
 */
NTMAP_PTR nxt_trait_vint (int sx, int mi, PRNTVMAP_PTR p_pr)
{
   int   i, j;
   VMAP_PTR p;
   NTMAP_PTR p_i = NULL;

   for (i = 1; !p_i && i <= 4; ++i)
   {
      if (i == 1)                       /* checks for gender-specific map     */
         j = sx;                        /* data present in this order         */
      else if (i == 2)
         j = AVG;
      else if (i == 3)
         j = FEM;
      else if (i == 4)
         j = MAL;
      if (p = first_trait_vmap_node[j])
      {                                 /* first finds chrom for this gender  */
         while (p->p_next && (p->chrom_no < p_pr->chrom_no))
            p = p->p_next;
         if (p->chrom_no == p_pr->chrom_no)
         {                              /* then looks for the interval        */
            p_i = p->p_intv;
            while (p_i && p_i->p_next && p_i->intv_no < mi)
               p_i = p_i->p_next;
            if (!(p_i && p_i->intv_no == mi))
               p_i = NULL;
         }
      }
   }
   return p_i;
}

/**
 * checks select marker and trait statements for errors.
 */
int test_atz_sel (void)
{
   int   n;
   UNLNKD_PTR p;

   if (!first_sim_node)                 /*select ... markers .. required      */
      err_typ = IBDDROP || MARKERDROP ? ERA_SLU : ERA_SEL;

   else if (first_sim_node->p_next)     /*checks for only one chromosome      */
      err_typ = ERA_OOS;

   else
   {                                    /*checks for the no. of mapped or     */
      n = first_sim_node->no_traits;    /*unlinked traits                     */

      if (p = first_unlnkd_node)
      {
         n++;
         while (p = p->p_next)
            n++;
      }
      /*                                 * these programs require one trait,
       *                                 * linked or unlinked                 */
      if (IBDDROP || LM_AUTO)
         if (!n)
            err_typ = IBDDROP ? ERA_OIT : ERA_OOT;

      /*                                 * no Auotozy/Lodscore program
       *                                 * permits more than one trait        */
      if (n > 1)
      {
         sprintf (errnumb, "%d", n);    /* these programs permit one trait    */
         err_txt = errnumb;
         err_typ = IBDDROP ? ERA_MIT : ERA_MOT;
      }
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks marker data statements for errors
 */
int test_markdat (SIM_PTR p_sel)
{
   int   i, j, k;
   MARKDAT_PTR p_mdt;
   LOCUS_PTR p_locus;
   int  *p_a;
   char *p_o;

   NObs = 0;
   if (!(p_mdt = first_markdat_node))   /* finds marker data for this chrom   */
   {
      err_typ = ERA_NMD;
      for_chrom (p_sel->chrom_no);
   }
   else
   {
      while (p_mdt->p_next && p_mdt->chrom_no < p_sel->chrom_no)
         p_mdt = p_mdt->p_next;
      if (p_mdt->chrom_no != p_sel->chrom_no)
      {
         err_typ = ERA_NMD;
         for_chrom (p_sel->chrom_no);
      }
      else
      {                                 /* checks no. of markers              */
         if ((p_sel->no_mark_map && p_mdt->no_marks != p_sel->no_mark_map)
             || (!p_sel->no_mark_map && p_mdt->no_marks != 1))
         {
            err_typ = ERA_BND;
            for_chrom (p_mdt->chrom_no);
         }
         else
            NObs = p_mdt->no_obs;
      }
   }
   if (err_typ)
      return FALSE;

   p_o = p_mdt->p_obs;                  /* tests marker alleles within range  */
   p_a = p_mdt->p_alls;                 /* ... for each observed individual   */
   for (i = 0; i < NObs; ++i)
   {
      p_locus = first_mark_locus_node;
      for (j = 0; j < p_sel->no_mark_map; ++j) /* ... and each marker    */
      {
         while ((p_locus->p_next && (p_locus->chrom_no < p_sel->chrom_no)
                 || (p_locus->chrom_no == p_sel->chrom_no && p_locus->no <= j)))
            p_locus = p_locus->p_next;

         for (k = 0; k < 2; ++k)
         {
            if (*p_a < 0 || *p_a > p_locus->no_alleles)
            {
               sprintf (errnumb, "%s", p_o);
               err_txt = errnumb;
               err_typ = ERA_WMA;
            }
            ++p_a;
         }
      }
      p_o += MAXNAMLEN + 1;
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks marker names statements for errors
 */
int test_marknam (SIM_PTR p_sel)
{
   MARKNAM_PTR p_mnm;

   if (!(p_mnm = first_marknam_node))   /* looks for marker names for this chrom  */
      return TRUE;
   else
   {
      while (p_mnm->p_next && p_mnm->chrom_no < p_sel->chrom_no)
         p_mnm = p_mnm->p_next;
      if (p_mnm->chrom_no != p_sel->chrom_no)
         return TRUE;
      else
         if ((p_sel->no_mark_map && p_mnm->no_marks != p_sel->no_mark_map)
             || (!p_sel->no_mark_map && p_mnm->no_marks != 1))
      {
         err_typ = ERA_BNM;
         for_chrom (p_mnm->chrom_no);
      }
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks program gamete statements for errors
 */
int test_probgam (SIM_PTR p_sel)
{
   PROBGAM_PTR p_pg;
   LOCWND_PTR p_node;

   if (!(p_pg = first_probgam_node))    /* expects proband gametes            */
      err_typ = ERA_MPG;

   else if (p_node = first_locwind_node) /* checks for locus window          */
   {                                    /* for this chromosome                */
      while (p_node->p_next && p_node->chrom_no < p_sel->chrom_no)
         p_node = p_node->p_next;

      if (p_node->chrom_no == p_sel->chrom_no)
      {
         NLocWndw = p_node->window;
         if (NLocWndw < 2 || NLocWndw > NLoci)
         {
            err_typ = ERA_LWO;
            sprintf (errnumb, "%d", NLoci);
            err_txt = errnumb;
         }
      }
   }
   if (IBDDROP && NLocWndw)             /* if ibddrop, and locus window,      */
      while (p_pg)                      /* tests for one pair of gametes      */
      {                                 /* in each set                        */
         if (p_pg->no_pgams != 2)
            err_typ = ERA_PIG;
         p_pg = p_pg->p_next;
      }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks the pattern label statements for errors.
 *
 * expects a locus window size if there are window patterns to score;
 * if not, a warnining is issued                                    
 * Expects at least one proband gamete scoreset for each component for which 
 * there are pattern labels; if not, a warning is issued                    
 * Each pattern label must be in the correct range, given the number of    
 * probands                                                               
 */
int test_patlab (SIM_PTR p_sel)
{
   int   i;

   PROBGAM_PTR p_pg;
   WNDPAT_PTR p_wp;

   if (!(p_wp = first_windpat_node))
      return TRUE;

   if (!NLocWndw)
   {
      err_typ = ERA_MLW;
      return FALSE;
   }

   p_pg = first_probgam_node;

   while (p_wp)
   {
      while ((p_pg->p_next && p_pg->component < p_wp->component)
             || (p_pg->p_next && p_pg->component == p_wp->component
                 && p_pg->scoreset < p_wp->scoreset))
         p_pg = p_pg->p_next;

      if (p_wp->component != p_pg->component
          || p_wp->scoreset != p_pg->scoreset)
      {
         err_typ = ERA_MSS;
         sprintf (errnumb, "component %d scoreset %d", p_wp->component,
                  p_wp->scoreset);
         err_txt = errnumb;
      }
      else
      {
         for (i = 1; i < p_wp->no_pats; ++i)
            if (*(p_wp->p_pats + i) > CDgg[p_pg->no_pgams])
            {
               err_typ = ERA_BPC;
               sprintf (errnumb, "%d", *(p_wp->p_pats + i));
               err_txt = errnumb;
            }
      }
      p_wp = p_wp->p_next;
   }

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks the list of affected individuals.
 */
int test_affind (void)
{
   if (!AffTrIn && !first_affnam_node)  /* affected individuals must be named */
      err_typ = ERA_NAF;                /* in par statement or ped file       */

   else if (AffTrIn && first_affnam_node) /* if named both in par and ped    */
   {                                    /* file, ped file phenos are ignored  */
      err_typ = ERA_BAF;
      AffTrIn = N;
   }
   if (err_typ)
      return FALSE;
   return TRUE;
}

int test_ps_pr (void)
/**
 * checks pseudo-priors
 */
{
   if (InPseudo)
      if (n_pseudo != NTPos + 1)
      {
         err_typ = ERA_NPP;
         sprintf (errnumb, "%d", NTPos + 1);
         err_txt = errnumb;
         return FALSE;
      }
      else if (PseudoPrior[0] == 0.0)
      {
         err_typ = ERA_ZUT;
         return FALSE;
      }
   return TRUE;
}

/**
 * checks scalar parameters and sets defaults
 */
int test_atz_pars (void)
{
   if (MARKERDROP)
      return TRUE;

   fprintf (stdout, " \n");

   if (NIt == UNDEF)
   {
      if (!LM_MAP)
      {
         err_typ = ERA_NIT;
         return FALSE;
      }
      else
         NIt = MCEMIT;
   }
   if (IBDDROP)
   {
      if (err_typ)
         return FALSE;
      return TRUE;
   }

   if (SmplSetup == UNDEF)
      SmplSetup = SEQIMP;

   if (NSeqImp == UNDEF)
   {
      NSeqImp = RSEQIMP * NIt;
      if (NSeqImp < 2)
         NSeqImp = 2;
   }

   if (Scan == UNDEF)
      Scan = Y;
   if (ScorIt == UNDEF)
   {
      if (!LM_MAP && !LM_IBDTESTS)
         ScorIt = 1;
      else if (LM_IBDTESTS)
         ScorIt = NIt / 50 > 10 ? 50 : NIt / 10;
   }

   if (SeqImpIt == UNDEF)
      SeqImpIt = 0;

   if (LM_BAYES)
      if (NPreIt == UNDEF)
         NPreIt = RPREIT * NIt;

   if (NBrnIt == UNDEF)
      if (LM_MAP)
         NBrnIt = RBRNIT_MAP * NIt;
      else if (LM_PVAL)
         NBrnIt = RBRNIT_PVAL * NIt;
      else if (LM_IBDTESTS)
         NBrnIt = RBRNIT_IBDTESTS * NIt;
      else if (LM_AUTO)
         NBrnIt = RBRNIT_AUTO * NIt;
      else if (LM_LODS)
         NBrnIt = RBRNIT_LODS * NIt;
      else if (LM_BAYES)
         NBrnIt = RBRNIT_BAYES * NPreIt;
      else if (LM_MARKERS)
         NBrnIt = RBRNIT_MRKS * NIt;
      else if (LM_TWOQTL)
         NBrnIt = RBRNIT_TWOQTL * NIt;
      else if (LM_SCHNELL)
         NBrnIt = RBRNIT_SCHNELL * NIt; /* FIXME:  doesn't use             */

   if (LM_MAP)
      if (MapTyp == UNDEF)
         MapTyp = BOTH;

   if (LM_PVAL)
   {
      if (SpairsOn == UNDEF && RobdomOn == UNDEF && MaxFGLOn == UNDEF)
      {
         MaxFGLOn = 1;                  /* assigns default values */
         SpairsOn = 0;
         RobdomOn = 0;
      }
   }

   if (LM_IBDTESTS)
      if (IBDStats == UNDEF && LikRatStats == UNDEF)
      {
         err_typ = ERA_NCS;
         return FALSE;
      }
      else if (IBDStats == 1)           /* checks IBD statistics options      */
      {
         LikRatStats = 0;
         if (DeltaOn != UNDEF || Lambda_pOn != UNDEF) /* first measures     */
            err_typ = ERA_WIM;
         DeltaOn = 0;
         Lambda_pOn = 0;

         if (NormalEnable == UNDEF || PermutEnable == UNDEF)
         {
            NormalEnable = 1;           /* assigns tests defaults             */
            PermutEnable = 0;
         }

         if (SpairsOn == UNDEF || RobdomOn == UNDEF || SAffUnaffOn == UNDEF
             || LambdaOn == UNDEF || MaxFGLOn == UNDEF)
         {
            SpairsOn = 1;               /* assigns meas defaults              */
            RobdomOn = 0;
            SAffUnaffOn = 0;
            LambdaOn = 0;
            MaxFGLOn = 0;
         }

         if (NormalEnable && SAffUnaffOn)
            err_typ = ERA_NSU;

         else if (ChiSqEnable != UNDEF) /* checks tests                       */
            err_typ = ERA_WIT;
         ChiSqEnable = 0;
      }

      else if (LikRatStats == 1)        /* checks lhood-ratio stat options    */
      {
         IBDStats = 0;
         if (SpairsOn != UNDEF || RobdomOn != UNDEF /* first measures         */
             || SAffUnaffOn != UNDEF || LambdaOn != UNDEF || MaxFGLOn != UNDEF)
            err_typ = ERA_WLM;
         SpairsOn = 0;
         RobdomOn = 0;
         SAffUnaffOn = 0;
         LambdaOn = 0;
         MaxFGLOn = 0;

         if (DeltaOn == UNDEF || Lambda_pOn == UNDEF)
         {
            DeltaOn = 1;                /* assigns meas defaults              */
            Lambda_pOn = 0;
         }
         if (NormalEnable != UNDEF || PermutEnable != UNDEF) /* checks tests  */
            err_typ = ERA_WLT;
         NormalEnable = 0;              /* assigns tests defaults             */
         PermutEnable = 0;

         if (ChiSqEnable = UNDEF)
            ChiSqEnable = 1;

         if (Lambda_pOn)                /* assigns grid spacing               */
         {
            IntervalSegL = .5 / (NumOfSegL - 1.);
            IntervalSegP = .8 / (NumOfSegP - 1.);
         }
      }

   if (ChkIt == UNDEF)
      ChkIt = NIt / 5;

   if (FilIt == UNDEF)
      FilIt = NIt;

   if (err_typ)
      return FALSE;
   return TRUE;
}

/**
 * checks and sets TraitTyp and PedDatTyp
 */
int test_trait_typ (void)
{
   if (IBDDROP || LM_MAP)
      return TRUE;

   if ((LM_PVAL || LM_IBDTESTS) && !AffTrIn)
      return TRUE;

   if (TraitTyp == UNDEF)
      if (LM_AUTO || LM_PVAL)
         TraitTyp = GENOTYPIC;
      else if (LM_IBDTESTS)
         TraitTyp = DISCRETE;
      else if (LM_MARKERS || LM_BAYES || LM_LODS)
         TraitTyp = GENOTYPIC;
      else if (LM_SCHNELL || LM_TWOQTL)
         TraitTyp = QUANTITATIVE;
      else if (MARKERDROP)
         if (PedDatTyp == UNDEF)
            TraitTyp = DISCRETE;
         else
            TraitTyp = PedDatTyp;

   if (PedDatTyp == UNDEF)
   {
      if (TraitTyp == GENOTYPIC || TraitTyp == DISCRETE
          || TraitTyp == QUANTITATIVE)
         PedDatTyp = TraitTyp;
   }
   else if (PedDatTyp == INHERITANCE && TraitTyp != PedDatTyp)
   {
      TraitTyp = PedDatTyp;
      err_typ = ERA_IPT;
      return FALSE;
   }

   return TRUE;
}

/**
 * checks pedigree record parameters for trait.
 * set pedigree data type.
 * tests for trait record positions compatible with no. of integers or reals.
 * allows for gender added by Pedchk/ function in pedigree checking step.
 */
int test_ped_trtdat (void)
{
   int   found = 0;
   int   nd = 1;
   int   tno;
   TRTPED_PTR p_p;

   if (IBDDROP || LM_MAP)
      return TRUE;

   if ((LM_PVAL || LM_IBDTESTS) && !AffTrIn)
      return TRUE;

   if (first_tlist_node)
      tno = first_tlist_node->no;       /*gets trait no. from select statemnt */
   else if (AffTrIn)                    /* gets trait no. from set affected   */
      tno = AffTrIn;                    /* individs statement                 */

   if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB)
      ++nd;

   if (p_p = first_trtped_node)         /* some trait record pos are given    */
   {
      while (p_p->p_next && p_p->no < tno) /* looks for record position      */
         p_p = p_p->p_next;             /* for this trait                     */

      if (p_p->no == tno)
      {
         found = TRUE;

         switch (PedDatTyp)
         {
             case QUANTITATIVE:
                if (p_p->dat_typ != REALDAT)
                {
                   err_typ = ERA_BRR;
                   sprintf (errnumb, "%d", tno);
                   err_txt = errnumb;
                }
                else if (p_p->pos[0] > NReals) /* checks pos against no reals */
                {
                   err_typ = ERA_NEK;
                   sprintf (errnumb, "%d", p_p->pos[0]);
                   err_txt = errnumb;
                }
                break;

             case INHERITANCE:
             case DISCR_LIAB:
                if (p_p->dat_typ != INTDAT2)
                {
                   err_typ = ERA_BI2;
                   sprintf (errnumb, "%d", tno);
                   err_txt = errnumb;
                }
                else if (p_p->pos[0] > NInts) /* checks pos against no. ints  */
                {
                   err_typ = ERA_NEJ;
                   sprintf (errnumb, "%d", p_p->pos[0]);
                   err_txt = errnumb;
                }
                else if (p_p->pos[1] > NInts) /* chks 2nd pos and no. ints    */
                {
                   err_typ = ERA_NEJ;
                   sprintf (errnumb, "%d", p_p->pos[1]);
                   err_txt = errnumb;
                }
                else if (p_p->pos[0] == p_p->pos[1])
                {
                   err_typ = ERA_DRP;
                   sprintf (errnumb, "%d", p_p->pos[0]);
                   err_txt = errnumb;
                }
                break;

             case GENOTYPIC:
             case DISCRETE:
             default:
                if (p_p->dat_typ != INTDAT)
                {
                   err_typ = ERA_BIR;
                   sprintf (errnumb, "%d", p_p->no);
                   err_txt = errnumb;
                }
                if (p_p->pos[0] > NInts) /* checks position against no. ints  */
                {
                   err_typ = ERA_NEJ;
                   sprintf (errnumb, "%d", p_p->pos[0]);
                   err_txt = errnumb;
                }
                break;
         }
         if (err_typ)
            return FALSE;
      }
      else                              /* didn't find this trait no.         */
      {
         switch (TraitTyp)
         {
             case QUANTITATIVE:
                err_typ = ERA_TER;
                break;
             case INHERITANCE:
                err_typ = ERA_TE2;
                break;
             default:
                err_typ = ERA_TEI;
                break;
         }
         sprintf (errnumb, "%d", tno);
         err_txt = errnumb;
      }
   }
   if (!found)                          /* found no record pos for this trait */
   {
      if (PedDatTyp != QUANTITATIVE)    /* note: SexIn may be UNDEF */
      {
         if (tno + (SexIn != N ? 1 : 0) > NInts - nd + 1)
         {
            err_typ = ERA_NEI;
            sprintf (errnumb, "%d", tno);
            err_txt = errnumb;
         }
      }
      else if (tno > NReals)
      {
         err_typ = ERA_NER;
         sprintf (errnumb, "%d", tno);
         err_txt = errnumb;
      }
   }
   if (err_typ)
      return FALSE;

   return TRUE;
}

/**
 * checks the map for trial trait positions.
 * checks for correct no. of traits. 
 * tests for trait map present for this chromosome.
 * checks sex-specific maps for consistency.
 * checks for marker no.'s within range
 */
int test_trait_vmap (SIM_PTR p_sel, int *ss_map)
{
   int   sx, j, beg = 0, end = 0, intv = 0;
   VMAP_PTR p_vmap;
   VMAP_PTR p_m[3] = { NULL, NULL, NULL };
   NTMAP_PTR p_n;

   if (p_sel->no_traits != 1)           /* check for one trait only           */
   {
      err_typ = ERA_NAT;
      return FALSE;
   }
   for (sx = FEM; sx <= AVG; ++sx)      /* check for trait map present        */
      if (p_vmap = first_trait_vmap_node[sx])
      {
         while (p_vmap->p_next && p_vmap->chrom_no < p_sel->chrom_no)
            p_vmap = p_vmap->p_next;
         if (p_vmap->chrom_no == p_sel->chrom_no)
         {
            p_m[sx] = p_vmap;           /* if found, check for sex-spec maps  */
            if (sx != AVG)
               *ss_map = TRUE;
            if (p_m[sx]->p_beg)
               beg = TRUE;
            if (p_m[sx]->p_end)
               end = TRUE;
            if (p_m[sx]->p_intv)
               intv = TRUE;
         }
      }
   if (!p_m[AVG] && !p_m[MAL] && !p_m[FEM]) /* check for at least 1 extern   */
   {                                    /* or interval prop map present       */
      if (p_sel->no_mark_sel > 1)       /* (need for trait #)                 */
         err_typ = ERA_MXM;             /* FIXME  ??? remove later ???        */
      else                              /* (sel trait statement)              */
         err_typ = ERA_MBM;             /* FIXME when adding trait no.        */
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }

   for (sx = MAL; sx <= AVG; ++sx)      /* checks for trait no. the same      */
   {
      j = (sx + 1) % (AVG + 1);
      if (p_m[sx] && p_m[j])
         if (p_m[sx]->no != p_m[j]->no)
            err_typ = ERD_TID;
   }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }

   if (p_sel->no_mark_sel == 1)         /* if only 1 marker,                  */
   {
      if (!beg || !end)                 /* test for extern positions missing  */
         err_typ = ERA_IMI;
      if (intv)                         /* and for extraneous interval props  */
         err_typ = ERA_XEI;
   }
   else if (p_sel->no_mark_sel > 1)     /* if more than 1 marker,             */
   {
      if (!beg && !end && !intv)        /* test for no trial trait positions  */
         err_typ = ERA_DAP;

      else if (!beg || !end || !intv)   /* and for some trait positions       */
         err_typ = ERA_DBE;             /* missing                            */
   }
   if (p_sel->no_mark_map > 1 && intv)  /* test for valid interval no.'s      */
      for (sx = MAL; sx <= AVG; ++sx)
         if (p_m[sx] && (p_n = p_m[sx]->p_intv))
         {
            while (p_n && p_n->p_next)
               p_n = p_n->p_next;
            if (p_n->intv_no > p_sel->no_mark_map - 1)
               err_typ = ERA_MIO;
         }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }

   return TRUE;
}

/**
 * sets mapping information for "selected" markers.
 * determines:
 *    whether this marker is selected
 *    the selected reference marker for mapping
 *    the dist from the selected marker
 */
void sel_mark_locus (SIM_PTR p_sel, PRNTLOC_PTR p_pl, int *p_entry)
/*
 * for this gender and this locus,  adds info to prntmap_locus structures
 */
{
   PRNTLOC_PTR p_cur;
   int   i;
   static int s_i;                      /* relative selected no.              */
   static int s_no;                     /* selected marker no.                */
   static int s_no1;                    /* 1st selected marker no.            */

   if (!*p_entry)                       /* tests for start of marker list     */
   {
      *p_entry = TRUE;
      s_i = 0;
      if (p_sel->sim_typ != AMK && p_sel->sim_typ != MK)
      {
         s_no1 = *(p_sel->p_mark);
         s_no = s_no1;
      }
   }

   if (p_sel->sim_typ == AMK || p_sel->sim_typ == MK) /* if all marks selctd, */
   {                                    /* copies marker and dist for mapping */
      p_pl->selectd = ++s_i;
      p_pl->sel_map_mark = p_pl->map_mark;
      p_pl->sel_dist = p_pl->dist;

      return;
   }

   p_pl->sel_map_mark = s_no;           /* if selectn, sets marker for mappng */

   p_pl->selectd = 0;                   /* sets selectn/non-selectn indicator */

   for (i = 0; i < p_sel->no_mark_sel; ++i)
      if (p_pl->no == *(p_sel->p_mark + i))
      {
         p_pl->selectd = ++s_i;
         s_no = p_pl->no;
      }

   if (p_pl->no < s_no1)                /* marker precedes 1st selectd markr: */
      ;                                 /* postpones setting sel dist         */

   else if (p_pl->no == s_no1)          /* 1st selected marker:               */
   {                                    /* sets sel dist to 0.                */
      p_pl->sel_dist = 0.;
      p_pl->sel_map_mark = 0;

      if (p_pl->no > 1)                 /* goes back and sets sel dists for   */
      {                                 /* markers preceding this one         */
         p_cur = p_pl;
         while (p_cur = p_cur->p_prev)
            p_cur->sel_dist = p_cur->p_next->sel_dist + p_cur->p_next->dist;
      }
   }
   else if (p_pl->no > s_no1)           /* marker follows 1st sel mark:       */
      if (p_pl->p_prev->selectd)        /* if last marker was selected,       */
         p_pl->sel_dist = p_pl->dist;   /* sets sel dist to map dist          */
      else
         p_pl->sel_dist = p_pl->dist    /* if last was not selected, sets sel */
            + p_pl->p_prev->sel_dist;   /* dist to map dist + prev sel dist   */

   return;
}

/**
 * sets mapping information for traits relative to the "selected" marker.
 * determines:
 *    the selected reference marker for mapping
 *    the dist from the referenced marker
 */
void sel_trait_locus (PRNTLOC_PTR p_pl)
{
   p_pl->selectd = 0;                   /* presently no selection for traits  */

   if (p_pl->map_mark == 0)             /* trait precedes 1st marker:         */
   {
      if (p_pl->p_next->selectd)        /* sets selected marker and dist      */
         p_pl->sel_map_mark = p_pl->p_next->no;
      else
         p_pl->sel_map_mark = p_pl->p_next->sel_map_mark;
      p_pl->sel_dist = p_pl->dist + p_pl->p_next->sel_dist;
   }

   else if (p_pl->p_prev)               /* trait follows 1st marker:          */
   {
      if (p_pl->p_prev->selectd)        /* preceding marker selected:         */
      {
         p_pl->sel_map_mark = p_pl->p_prev->no;
         p_pl->sel_dist = p_pl->dist;   /* sets sel dist to map dist          */
      }
      else if (p_pl->p_prev->sel_map_mark < p_pl->p_prev->no) /* some prev   */
      {                                 /* marker selected:                   */
         p_pl->sel_map_mark = p_pl->p_prev->sel_map_mark; /* adds sel dist   */
         p_pl->sel_dist = p_pl->dist + p_pl->p_prev->sel_dist; /* to map dst */
      }
      else if (p_pl->p_prev->sel_map_mark > p_pl->p_prev->no) /* no prev     */
      {                                 /* marker selected                    */
         p_pl->sel_map_mark = p_pl->p_prev->sel_map_mark; /* subtrcts map dst */
         p_pl->sel_dist = p_pl->p_prev->sel_dist - p_pl->dist; /* from sel    */
      }
   }
   return;
}

/**
 * removes prntmap locus nodes for non-selected markers.
 * adjusts forward and backward ptrs 
 */
void sel_prntmap_mark (PRNTMAP_PTR p_pm)
{
   PRNTLOC_PTR p, p_sav;

   while ((p = p_pm->p_locus) && (p->locus_typ == MD) && (!p->selectd))
   {                                    /* looks for start of list            */
      p_pm->p_locus = p->p_next;        /* advances ptr to first locus        */
      free (p);                         /* removes this locus                 */
   }
   p->p_prev = NULL;

   p_sav = p;                           /* checks loci for deletion           */
   while (p = p_sav->p_next)
   {
      if (p->locus_typ == MD && !p->selectd)
      {
         p_sav->p_next = p->p_next;     /* advances last locus' forward ptr   */
         free (p);                      /* removes current locus              */
      }
      else
      {
         p->p_prev = p_sav;             /* updates current locus' backwrd ptr */
         p_sav = p;                     /* keeps current locus                */
      }
   }
   return;
}

/**
 * makes vprntmap node.
 * note: Autozyg programs use only one chromosome 
 */
PRNTVMAP_PTR add_vprntmap (VMAP_PTR p_vm, int sx)
{
   PRNTVMAP_PTR p_p;

   p_p = malloc (sizeof (PRNTVMAP));
   alloc_chk (WHERE, p_p);

   first_vpmap[sx] = p_p;
   p_p->chrom_no = p_vm->chrom_no;
   p_p->no = p_vm->no;
   p_p->p_beg = NULL;
   p_p->p_end = NULL;
   p_p->p_intv = NULL;
   p_p->p_sintv = NULL;

   return p_p;
}

/**
 * makes a list of external trial trait positions.
 * makes the vprntmap list for positions at the start or end of the chrom      
 */
void add_trait_vext (PRNTVMAP_PTR p_pv, int sex)
{
   XTMAP_PTR p_x;
   PXMAP_PTR p_px;

   int   i, k;

   for (k = BDM; k <= EDM; ++k)         /* sets beginning pos, then ending    */
   {
      p_px = malloc (sizeof (PXMAP));
      alloc_chk (WHERE, p_px);
      p_px->no_pos = 0;
      p_px->p_dist = NULL;
      p_px->p_rf = NULL;

      if (k == BDM)
         p_pv->p_beg = p_px;
      else
         p_pv->p_end = p_px;

      if (p_x = nxt_trait_vext (sex, p_pv, k)) /* looks for beg or endng map */
      {
         p_px->no_pos = p_x->no_pos;    /* uses map from selected gender      */
         p_px->p_dist = malloc (p_x->no_pos * sizeof (double));
         alloc_chk (WHERE, p_px->p_dist);
         p_px->p_rf = malloc (p_x->no_pos * sizeof (double));
         alloc_chk (WHERE, p_px->p_rf);
         for (i = 0; i < p_x->no_pos; ++i)
         {
            *(p_px->p_dist + i) = *(p_x->p_dist + i);
            *(p_px->p_rf + i) = *(p_x->p_rf + i);
         }
      }
      else
      {
         p_px->no_pos = NXtrnPos;       /* uses canned defaults for map       */
         p_px->p_dist = malloc (NXtrnPos * sizeof (double));
         alloc_chk (WHERE, p_px->p_dist);
         p_px->p_rf = malloc (NXtrnPos * sizeof (double));
         alloc_chk (WHERE, p_px->p_rf);
         for (i = 0; i < NXtrnPos; ++i)
            if (k == BDM)
            {
               *(p_px->p_rf + i) = *(XtrnPos + NXtrnPos - i - 1);
               *(p_px->p_dist + i) = rf2dist (*(XtrnPos + NXtrnPos - i - 1));
            }
            else if (k == EDM)
            {
               *(p_px->p_rf + i) = *(XtrnPos + i);
               *(p_px->p_dist + i) = rf2dist (*(XtrnPos + i));
            }
      }
   }
   return;
}

/**
 * makes a list of trial trait positions for ea marker interval.
 */
void add_trait_vint (PRNTVMAP_PTR p_pv, PRNTMAP_PTR p_pm, int sex)
{
   NTMAP_PTR p_n;
   PIMAP_PTR p_i, p_j;
   PRNTLOC_PTR p_l;
   int   i, m;

   p_l = p_pm->p_locus;

   for (m = 1; m < p_pm->no_mark_map; ++m)
   {
      p_j = malloc (sizeof (PIMAP));    /* allocs and initializes the node    */
      alloc_chk (WHERE, p_j);

      p_j->intv_no = m;
      p_j->selectd = p_l->selectd;
      p_j->no_prop = 0;
      p_j->p_prop = NULL;
      p_j->p_next = NULL;

      if (!(p_n = nxt_trait_vint (sex, m, p_pv))) /* looks for intv props    */
         p_n = nxt_trait_vint (sex, 0, p_pv); /* for this interval or "all"  */
      if (p_n)                          /* intervals, using default genders   */
      {
         p_j->no_prop = p_n->no_prop;
         p_j->p_prop = malloc (p_n->no_prop * sizeof (double));
         alloc_chk (WHERE, p_j->p_prop);
         for (i = 0; i < p_n->no_prop; ++i)
            *(p_j->p_prop + i) = *(p_n->p_prop + i);
      }
      else
      {
         p_j->no_prop = NIntProp;       /* uses canned defaults for map       */
         p_j->p_prop = malloc (NIntProp * sizeof (double));
         alloc_chk (WHERE, p_j->p_prop);
         for (i = 0; i < NIntProp; ++i)
            *(p_j->p_prop + i) = *(IntProp + i);
      }
      p_l = p_l->p_next;

      if (m == 1)
         p_pv->p_intv = p_j;
      else
         p_i->p_next = p_j;
      p_i = p_j;
   }
   return;
}

/**
 * builds a list of trait positions for ea selected marker interval,
 * and for the start and end of the chromosome.
 * all trait positions and non-selected markers are preserved; the
 * map positions are determined relative to the selected markers
 */
void sel_trait_pos (PRNTVMAP_PTR p_pv, PRNTMAP_PTR p_pm)
{
   int   i, k, j;
   double s;
   PIMAP_PTR p_i, pi_sav;
   PSMAP_PTR p_new;
   PRNTLOC_PTR p_l;

   p_pv->p_sintv = alloc_sintv (0);     /* allocs a node for ea selected      */
   p_new = p_pv->p_sintv;               /* interval, and the beginning & end  */

   for (i = 1; i <= p_pm->no_mark_sel; ++i)
   {
      p_new->p_next = alloc_sintv (i);
      p_new = p_new->p_next;
   }
   p_new->p_next = NULL;

   p_i = p_pv->p_intv;                  /* BEGINNING:  counts no. positions   */
   p_new = p_pv->p_sintv;               /* and allocates list                 */
   p_new->no_pos = p_pv->p_beg->no_pos;
   while (p_i && !p_i->selectd)
   {
      p_new->no_pos += p_i->no_prop + 1;
      p_i = p_i->p_next;
   }
   p_i = p_pv->p_intv;
   p_l = p_pm->p_locus;

   alloc_sintl (p_new);

   s = p_l->sel_dist;                   /* sets positions before marker 1     */

   for (k = 0; k < p_pv->p_beg->no_pos; ++k)
   {
      *(p_new->p_sdist + k) = *(p_pv->p_beg->p_dist + k) + s;
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      if (k < p_pv->p_beg->no_pos - 1)
         *(p_new->p_locdist + k) = *(p_pv->p_beg->p_dist + k)
            - *(p_pv->p_beg->p_dist + k + 1);
      else
         *(p_new->p_locdist + k) = *(p_pv->p_beg->p_dist + k);
   }

   p_i = p_pv->p_intv;                  /* for ea proportion list before 1st  */
   while (p_i && !p_i->selectd)         /* sel marker, sets pos to marker pos */
   {                                    /* and to interval positions          */
      *(p_new->p_sdist + k) = s;
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      *(p_new->p_star + k - 1) = TRUE;
      s = p_l->p_next->sel_dist;

      for (j = 0; j < p_i->no_prop; ++j, ++k)
      {
         *(p_new->p_sdist + k + 1) = s + (1. - *(p_i->p_prop + j))
            * p_l->p_next->dist;
         *(p_new->p_rf + k + 1) = dist2rf (*(p_new->p_sdist + k + 1));
         *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
            - *(p_new->p_sdist + k + 1);
      }
      *(p_new->p_locdist + k) = *(p_new->p_sdist + k) - s;
      ++k;

      p_i = p_i->p_next;
      p_l = p_l->p_next;
   }
   p_new->sel_map_mark = p_l->no;

   for (i = 1; i < p_pm->no_mark_sel; ++i) /* INTERVALS: for ea interval     */
   {                                    /* between selected markers, counts   */
      pi_sav = p_i;                     /* no. positions and allocs list      */
      p_new = p_new->p_next;
      p_new->no_pos = p_i->no_prop;
      while ((p_i = p_i->p_next) && !p_i->selectd)
         p_new->no_pos += p_i->no_prop + 1;
      p_new->sel_map_mark = p_l->no;

      alloc_sintl (p_new);

      p_i = pi_sav;
      s = 0.;
      k = 0;

      do                                /* for this proportn list, sets pos   */
      {
         for (j = 0; j < p_i->no_prop; ++j, ++k)
         {
            *(p_new->p_sdist + k) = s +
               *(p_i->p_prop + j) * (p_l->p_next->dist);
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
            if (j == 0)
               *(p_new->p_locdist + k) = *(p_i->p_prop) * (p_l->p_next->dist);
            else
               *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
                  - *(p_new->p_sdist + k - 1);
         }
         p_l = p_l->p_next;
         p_i = p_i->p_next;
         s = p_l->sel_dist;
         *(p_new->p_locdist + k) = s - *(p_new->p_sdist + k - 1);

         if (p_i && !p_i->selectd)      /* sets pos for non-selectd marker    */
         {
            *(p_new->p_sdist + k) = s;
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
            *(p_new->p_star + k) = TRUE;
            ++k;
         }
      }
      while (p_i && !p_i->selectd);
   }

   p_new = p_new->p_next;               /* ENDING: counts no. positions       */
   pi_sav = p_i;                        /* and allocates list                 */
   p_new->no_pos = 0;

   if (p_l->no < p_pm->no_mark_map)
      while (p_i)
      {
         p_new->no_pos += p_i->no_prop + 1;
         p_i = p_i->p_next;
      }
   p_new->no_pos += p_pv->p_end->no_pos;
   p_new->sel_map_mark = p_l->no;

   alloc_sintl (p_new);

   p_i = pi_sav;
   k = 0;
   s = 0.;
   while (p_i)                          /* for ea remaining prop list between */
   {                                    /* non-selectd markers, sets positns  */
      for (j = 0; j <= p_i->no_prop; ++j, ++k) /* and 1 for marker pos       */
      {
         if (j < p_i->no_prop)
         {
            *(p_new->p_sdist + k) = s + *(p_i->p_prop + j) * p_l->p_next->dist;
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
         }
         else
         {
            s = p_l->p_next->sel_dist;
            *(p_new->p_sdist + k) = s;
            *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
            *(p_new->p_star + k) = TRUE;
         }
         if (k == 0)
            *(p_new->p_locdist + k) = *(p_new->p_sdist + k);
         else
            *(p_new->p_locdist + k) = *(p_new->p_sdist + k)
               - *(p_new->p_sdist + k - 1);
      }
      p_l = p_l->p_next;
      p_i = p_i->p_next;
   }

   for (j = 0; j < p_pv->p_end->no_pos; ++j, ++k) /* sets pos after last m   */
   {
      *(p_new->p_sdist + k) = s + *(p_pv->p_end->p_dist + j);
      *(p_new->p_rf + k) = dist2rf (*(p_new->p_sdist + k));
      if (j == 0)
         *(p_new->p_locdist + k) = *(p_pv->p_end->p_dist);
      else
         *(p_new->p_locdist + k) = *(p_pv->p_end->p_dist + j)
            - *(p_pv->p_end->p_dist + j - 1);
   }
   return;
}

/*
 * removes non-selected loci from list of loci
 */
void shrink_prntmap (PRNTMAP_PTR p_pmap)
{
   PRNTLOC_PTR p_cur, p_nxt, p_sav;

   p_cur = p_pmap->p_locus;
   while (p_cur && p_cur->locus_typ == MD && !p_cur->selectd)
   {
      p_nxt = p_cur->p_next;
      free (p_cur);
      p_cur = p_nxt;
   }
   p_pmap->p_locus = p_cur;
   p_sav = p_cur;
   if (p_cur)                           /* 1st selected marker          */
   {
      p_cur->p_prev = NULL;
      p_cur->dist = p_cur->sel_dist;
   }

   while (p_cur)
   {
      if (p_cur->locus_typ == MD && !p_cur->selectd)
      {
         p_sav->p_next = p_cur->p_next;
         p_nxt = p_cur->p_next;
         free (p_cur);
         p_cur = p_nxt;
         if (p_cur)
            p_cur->p_prev = p_sav;
      }
      else
      {
         p_sav = p_cur;                 /* next selected marker        */
         p_cur->dist = p_cur->sel_dist;
         p_cur = p_cur->p_next;
      }
   }
}

PSMAP_PTR alloc_sintv (int i)
{
   PSMAP_PTR p;

   p = malloc (sizeof (PSMAP));
   alloc_chk (WHERE, p);
   p->intv_no = i;
   p->p_sdist = NULL;
   p->p_rf = NULL;
   p->p_locdist = NULL;
   p->p_star = NULL;
   p->p_next = NULL;

   return p;
}

void alloc_sintl (PSMAP_PTR p_s)
{
   int   i;

   p_s->p_sdist = malloc (p_s->no_pos * sizeof (double));
   p_s->p_rf = malloc (p_s->no_pos * sizeof (double));
   p_s->p_locdist = malloc ((p_s->no_pos + 1) * sizeof (double));
   p_s->p_star = malloc (p_s->no_pos * sizeof (int));
   for (i = 0; i < p_s->no_pos; ++i)
      *(p_s->p_star + i) = 0;

   return;
}

/**
 * checks the beginning and ending trial trait positions for consistency.
 */
int test_trait_vext (SIM_PTR p_sel)
{
   if (first_vpmap[FEM]->p_beg->no_pos != first_vpmap[MAL]->p_beg->no_pos)
      err_typ = ERA_WDX;
   else if (first_vpmap[FEM]->p_end->no_pos != first_vpmap[MAL]->p_end->no_pos)
      err_typ = ERA_WDX;
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

/**
 * checks the trait positions in the marker intervals for consistency 
 */
int test_trait_vint (SIM_PTR p_sel)
/*                                      * checks intra-interval trait pos     *
 *                                      * for consistency                     */
{
   PIMAP_PTR p_f, p_m;

   p_f = first_vpmap[FEM]->p_intv;
   p_m = first_vpmap[MAL]->p_intv;

   while (p_f)
   {
      if (p_f->no_prop != p_m->no_prop)
         err_typ = ERA_WIP;
      p_f = p_f->p_next;
      p_m = p_m->p_next;
   }
   if (err_typ)
   {
      for_chrom (p_sel->chrom_no);
      return FALSE;
   }
   return TRUE;
}

void prnt_vtrait_freqs (SIM_PTR p_sel)
{
   char  prnt_lin[81];

   if (!nxt_trait_vmap (FEM, p_sel))
      return;

   fprintf (stdout,                     /* formats the frequencies for the    */
            " Trait allele frequencies:\n\n"); /* lods trait                 */
   fmt_trait_freqs (prnt_lin, 0, nxt_trait_vmap (FEM, p_sel)->no);

   fprintf (stdout, "%s\n\n", prnt_lin); /* prints the line                  */
   fflush (stdout);

   return;
}

int test_penetrnc (void)
{
   if (NTraits && TraitTyp == DISCRETE && PedDatTyp != DISCR_LIAB)
      if (NPenVals == 0)
      {
         err_typ = ERA_MPV;
         return FALSE;
      }
   if (MARKERDROP && PedDatTyp == INHERITANCE)
   {
      NPenVals = 0;
      err_typ = ERA_PNI;
      return FALSE;
   }

   return TRUE;
}

void prnt_penetrnc (void)
{
   if (!NPenVals)
      return;

   fprintf (stdout, " Incomplete penetrances for the trait are: \n");
   fprintf (stdout, "        genotype  code  penetrance\n");
   fprintf (stdout, "        --------  ----  ----------\n");
   fprintf (stdout, "           (1 1)     0       %5.3f\n", PenVal[0]);
   fprintf (stdout, "    (1 2), (2 1)     1       %5.3f\n", PenVal[1]);
   fprintf (stdout, "           (2 2)     2       %5.3f\n\n", PenVal[2]);

   return;
}

void prnt_qtrait_pars (SIM_PTR p_sel)
{
   char  prnt_lin[81];

   if (!nxt_trait_vmap (FEM, p_sel))
      return;

   fprintf (stdout, " Genotype means                        "
            " Additive, residual variances\n\n");

   fmt_trait_pars (prnt_lin, nxt_trait_vmap (FEM, p_sel)->no);

   fprintf (stdout, "%s\n", prnt_lin);
   fprintf (stdout, "\n");
   fflush (stdout);

   return;
}

/**
 * allocates and sets MarkNam[][] which contains the names of the
 * selected markers, either provided names or default names.
 * returns true if marker names given in par statements.
 */
int mak_marknam (SIM_PTR p_sel)
{
   MARKNAM_PTR p_mnm;
   int   i, no, nc;
   char *p_n = NULL;

   if (!NMarks)
      return FALSE;

   MarkNam = malloc (NMarks * sizeof (char *));
   alloc_chk (WHERE, MarkNam);
   for (i = 0; i < NMarks; ++i)
   {
      MarkNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, MarkNam[i]);
   }

   if (p_mnm = first_marknam_node)      /* checks for names given for chrom   */
   {
      while (p_mnm->p_next && p_mnm->chrom_no < p_sel->chrom_no)
         p_mnm = p_mnm->p_next;
      if (p_mnm->chrom_no == p_sel->chrom_no)
         p_n = p_mnm->p_nams;
   }

   for (i = 0; i < NMarks; ++i)
   {
      if (!p_sel->p_mark)
         no = i + 1;
      else
         no = *(p_sel->p_mark + i);
      if (p_n)
      {
         if ((nc = get_marknam (MarkNam[i], p_n, no)) > LocNamLen)
            LocNamLen = nc;
      }
      else if ((nc = fmt_marknam (MarkNam[i], no)) > LocNamLen)
         LocNamLen = nc;
   }
   if (p_n)
      return TRUE;
   else
      return FALSE;
}

/**
 * allocates and sets TraitNam[][], which contains the name
 * of the selected trait.
 */
void mak_traitnam (TLIST_PTR p_tr)
{
   int   i, nc;

   if (!NTraits)
      return;

   TraitNam = malloc (NTraits * sizeof (char *));
   alloc_chk (WHERE, TraitNam);

   for (i = 0; i < NTraits; ++i)
   {
      TraitNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, TraitNam[i]);
   }
   for (i = 0; i < NTraits; ++i)
   {
      if ((nc = fmt_traitnam (TraitNam[i], p_tr->no)) > LocNamLen)
         LocNamLen = nc;
   }
   return;
}

/**
 * allocates global arrays for marker positions
 *   Marek:   2 sexes, no. markers recomb fracts 
 *            plus beginning and ending unlinked pos
 *   MarkPos: 2 sexes, no. marker positions
 */
void alloc_marek (void)
{
   int   sx;

   Marek = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, Marek);
   MarkPos = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, MarkPos);

   for (sx = FEM; sx <= MAL; ++sx)
   {
      Marek[sx] = malloc ((NMarks + 1) * sizeof (double));
      alloc_chk (WHERE, Marek[sx]);
      MarkPos[sx] = malloc (NMarks * sizeof (double));
      alloc_chk (WHERE, MarkPos[sx]);
   }
   return;
}

/**
 * allocates global arrays for trial trait positions.
 *    IntVal:   no. trait pos + 2
 *    RecVal:   2 sexes, no. trait pos + 2,  recomb fracts
 *    NTPosInt: no. intervals (no. markers + 1, for beginning, 
 *                             between markers and ending)
 */
void alloc_intval (void)
{
   int   sx;

   RecVal = malloc (2 * sizeof (double *));
   alloc_chk (WHERE, RecVal);

   for (sx = FEM; sx <= MAL; ++sx)
   {
      RecVal[sx] = malloc ((NTPos + 2) * sizeof (double));
      alloc_chk (WHERE, RecVal[sx]);
   }
   IntVal = malloc ((NTPos + 2) * sizeof (int));
   alloc_chk (WHERE, IntVal);

   NTPosInt = malloc ((NMarks + 1) * sizeof (int));
   alloc_chk (WHERE, NTPosInt);

   return;
}

/**
 * stores recombination fractions for markers.
 * makes Marek[][] : sex-specific recombination fractions
 *                   (1st and last one set to .5)
 */
void mak_marek (void)
{
   int   sx, entry, i;
   double pos;
   PRNTMAP_PTR p_pm;
   PRNTLOC_PTR p_loc;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      entry = FALSE;
      if (p_pm = nxt_prntmap (&entry, sx))
      {
         p_loc = p_pm->p_locus;

         pos = 0.;
         MarkPos[sx][0] = pos;
         Marek[sx][0] = .5;             /* ignores trait, if linked           */
         for (i = 1; p_loc; p_loc = p_loc->p_next) /* also 1st marker         */
            if (p_loc->locus_typ == MD && p_loc->dist)
            {
               pos += p_loc->dist;
               MarkPos[sx][i] = pos;
               Marek[sx][i++] = dist2rf (p_loc->dist);
            }
         Marek[sx][i] = .5;
      }
   }
   return;
}

/**
 * stores recombination fractions for traits.
 * makes Int0, Rec0[2] - trait position and sex-specific recomb fraction
 */
void mak_tarek (void)
{
   int   sx, entry, i, j;
   PRNTMAP_PTR p_pm;
   PRNTLOC_PTR p_loc;

   if (!NLnkd)
   {
      Loc0 = 0;
      Rec0[0] = .5;
      Rec0[1] = .5;
      TraitPos[0] = .5;
      TraitPos[1] = .5;
   }
   else
      for (sx = FEM; sx <= MAL; ++sx)
      {
         entry = FALSE;
         if (p_pm = nxt_prntmap (&entry, sx))
         {
            p_loc = p_pm->p_locus;
            for (i = 0, j = 0; p_loc; p_loc = p_loc->p_next)
            {
               if (p_loc->locus_typ == TD)
               {
                  Loc0 = j;
                  Rec0[sx] = dist2rf (p_loc->dist);
                  TraitPos[sx] = p_loc->dist;
               }
               else if (p_loc->locus_typ == MD && p_loc->selectd)
                  ++j;
            }
         }
      }
   return;
}

/**
 * stores marker and trial trait positions.
 *    NLocPos - total no. of marker and test trait positions
 *    LocPos  - data structure containing locus type, interval no.
 *              marker or trait no., recomb fract, distance from
 *              marker 1
 */
void mak_locpos (void)
{
   int   sx, i, tno, entry;
   double dm[2] = { 0., 0. };
   PSMAP_PTR p_tr[2];
   PRNTLOC_PTR p_mk[2];
   LOCPOS_PTR p_l;

   NLocPos = NTPos + NMarks;

   p_l = malloc (NLocPos * sizeof (LOCPOS));
   alloc_chk (WHERE, p_l);
   LocPos = p_l;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      p_tr[sx] = first_vpmap[sx]->p_sintv;
      entry = FALSE;
      p_mk[sx] = nxt_prntmap (&entry, sx)->p_locus;
   }
   tno = first_vpmap[FEM]->no;

   while (p_tr[FEM])
   {
      for (i = 0; i < p_tr[FEM]->no_pos; ++i)
      {
         p_l->locus_typ = TD;           /* for this interval, stores all      */
         p_l->intv_no = p_tr[FEM]->intv_no; /* trait recomb fracts and       */
         p_l->no = tno;                 /* positions for both sexes           */
         for (sx = FEM; sx <= MAL; ++sx)
         {
            p_l->rf[sx] = *(p_tr[sx]->p_rf + i);
            if (p_l->intv_no == 0)
               p_l->pos[sx] = -*(p_tr[sx]->p_sdist + i);
            else
               p_l->pos[sx] = dm[sx] + *(p_tr[sx]->p_sdist + i);
         }
         ++p_l;
      }
      for (sx = FEM; sx <= MAL; ++sx)
         p_tr[sx] = p_tr[sx]->p_next;

      if (p_mk[FEM])
      {
         p_l->locus_typ = MD;           /* for the marker at the end          */
         p_l->intv_no = p_mk[FEM]->selectd; /* of the interval,               */
         p_l->no = p_mk[FEM]->no;       /* stores the recomb fract            */
         for (sx = FEM; sx <= MAL; ++sx) /* and position for both sexes       */
         {
            p_l->rf[sx] = dist2rf (p_mk[sx]->sel_dist);
            if (p_mk[sx]->selectd == 1)
               p_l->pos[sx] = 0.;
            else
            {
               dm[sx] += p_mk[sx]->sel_dist;
               p_l->pos[sx] = dm[sx];
            }
            p_mk[sx] = p_mk[sx]->p_next;
         }
         ++p_l;
      }
   }
   return;
}

/**
 * stores trial trait positions.
 *    NTPosInt[] - list of no. trait positions in ea marker interval
 *    IntVal[]   - the interval no. corresponding to ea trait pos.
 */
void mak_intval (void)
{
   int   i, j, ix;
   PSMAP_PTR p_s;

   p_s = first_vpmap[FEM]->p_sintv;
   IntVal[0] = 0;
   ix = 1;

   for (j = 0; j <= NMarks && p_s; ++j)
   {
      NTPosInt[j] = p_s->no_pos;        /* sets no. positions in ea interval  */
      for (i = 0; i < p_s->no_pos; ++i, ++ix)
         IntVal[ix] = j;                /* sets interval for ea trait positn  */
      p_s = p_s->p_next;
   }
   IntVal[NTPos + 1] = 0;

   return;
}

/**
 * stores trial trait recombinaton fractions.
 * makes RecVal[][] - sex-specific recombination fracts corresp to IntVal[]
*/
void mak_recval (void)
{
   PSMAP_PTR p_s;
   int   sx, i, j, ix;

   for (sx = FEM; sx <= MAL; ++sx)
   {
      p_s = first_vpmap[sx]->p_sintv;
      RecVal[sx][0] = .5;               /* sets rec fracts for start of chrom */

      ix = 1;
      for (j = 0; j <= NMarks && p_s; ++j) /* and for ea marker interval     */
      {
         for (i = 0; i < p_s->no_pos; ++i, ++ix)
            RecVal[sx][ix] = *(p_s->p_rf + i);
         p_s = p_s->p_next;
      }
      RecVal[sx][NTPos + 1] = .5;       /* and for end of chrom               */
   }
   return;
}

/**
 * allocates arrays and stores allele frequencies for all selected loci.
 * on first entry, mallocs NAll[NLoci] and Qq[NLoci] (NLoci is NMarks+1)
 *    NAll: 0 .. NLoci-1
 *    Qq: 1st subscript:  0 .. NLoci-1
 *    Qq: 2nd: 1 .. NAll[1st subscript]
 *    (Note:  NAll[0], Qq[0] are trait)
 */
void mak_allefrq (PRNTMAP_PTR p_pmap, PRNTLOC_PTR p_ploc, int no)
{
   int   j, flg;
   double sum;
   static int m;

   LOCUS_PTR p_locus;

   if (!Qq)
   {
      NAll = malloc (NLoci * sizeof (int));
      alloc_chk (WHERE, NAll);
      Qq = malloc (NLoci * sizeof (double *));
      alloc_chk (WHERE, Qq);
      m = 1;
   }

   if (p_ploc && p_ploc->locus_typ == MD) /* sets no. alleles and stores     */
   {                                    /* allele freqs for marker            */
      p_locus = first_mark_locus_node;
      while ((p_locus->p_next
              && (p_locus->chrom_no < p_pmap->chrom_no)
              || (p_locus->chrom_no == p_pmap->chrom_no
                  && p_locus->no < p_ploc->no)))
         p_locus = p_locus->p_next;

      NAll[m] = p_locus->no_alleles;

      Qq[m] = malloc ((NAll[m] + 1) * sizeof (double));
      alloc_chk (WHERE, Qq[m]);

      Qq[m][0] = 0.;                    /* 1st element is unused              */
      flg = norm_frq (norm_frq_spec, p_locus->p_freq_node->sum);
      sum = p_locus->p_freq_node->sum;
      for (j = 0; j < NAll[m]; ++j)
      {
         Qq[m][j + 1] = *(p_locus->p_freq_node->p_freq + j);
         if (flg)                       /* normalizes freqs, if required      */
            Qq[m][j + 1] /= sum;
      }
      ++m;
   }
   else if (no == 0)                    /* no trait locus; still need Qq[][]  */
   {
      NAll[0] = 2;

      Qq[0] = malloc (3 * sizeof (double));
      alloc_chk (WHERE, Qq[0]);

      Qq[0][0] = 0.;                    /* 1st element is unused              */
      Qq[0][1] = .5;
      Qq[0][2] = .5;
   }
   else
   {                                    /* sets no. alleles and stores        */
      p_locus = first_trait_locus_node; /* allele freqs for trait             */
      while (p_locus->p_next && p_locus->no < no)
         p_locus = p_locus->p_next;

      NAll[0] = p_locus->no_alleles;

      Qq[0] = malloc ((NAll[0] + 1) * sizeof (double));
      alloc_chk (WHERE, Qq[0]);

      Qq[0][0] = 0.;                    /* 1st element is unused              */
      flg = norm_frq (norm_frq_spec, p_locus->p_freq_node->sum);
      sum = p_locus->p_freq_node->sum;
      for (j = 0; j < NAll[0]; ++j)
      {
         Qq[0][j + 1] = *(p_locus->p_freq_node->p_freq + j);
         if (flg)                       /* normalizes freqs, if required      */
            Qq[0][j + 1] /= sum;
      }
   }
   return;
}

void mak_markdat (SIM_PTR p_sel)
{
   int   i, j, k;
   int  *p_a;
   char *p_o;
   MARKDAT_PTR p_mdt;

   if (NObs == 0)
      return;

   if (!(p_mdt = first_markdat_node))   /* finds marker data for this chrom   */
      return;

   while (p_mdt->p_next && p_mdt->chrom_no < p_sel->chrom_no)
      p_mdt = p_mdt->p_next;
   if (p_mdt->chrom_no != p_sel->chrom_no)
      return;

   ObsNam = malloc (NObs * sizeof (char *)); /* mallocs:                     */
   alloc_chk (WHERE, ObsNam);           /*        ObsNam [NObs][MAXNAMLEN+1]  */
   ObsIx = malloc (NObs * sizeof (int)); /*      ObsIx  [NObs]               */
   alloc_chk (WHERE, ObsIx);
   ObsMark = malloc (NObs * sizeof (int **)); /* ObsMark[NObs][NMarks][2]    */
   alloc_chk (WHERE, ObsMark);

   for (i = 0; i < NObs; ++i)
   {
      ObsNam[i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
      alloc_chk (WHERE, ObsNam[i]);
      ObsMark[i] = malloc (NMarks * sizeof (int *));
      alloc_chk (WHERE, ObsMark[i]);
      for (j = 0; j < NMarks; ++j)
      {
         ObsMark[i][j] = malloc (2 * sizeof (int));
         alloc_chk (WHERE, ObsMark[i][j]);
      }
   }

   p_o = p_mdt->p_obs;
   p_a = p_mdt->p_alls;
   for (i = 0; i < NObs; ++i)           /* stores indices, names, alleles     */
   {
      ObsIx[i] = 0;
      strncpy (ObsNam[i], p_o, MAXNAMLEN + 1);
      p_o += MAXNAMLEN + 1;

      for (j = 0, k = 1; k <= (p_sel->no_mark_map ?
                               p_sel->no_mark_map : 1); ++k)
      {
         if (!p_sel->p_mark ||
             (j < p_sel->no_mark_sel && k == *(p_sel->p_mark + j)))
         {
            ObsMark[i][j][0] = *p_a;
            ObsMark[i][j][1] = *(p_a + 1);
            ++j;
         }
         p_a = p_a + 2;
      }
   }
   return;
}

void mak_probgam (void)
{
   int   c, s, i = 0;
   PROBGAM_PTR p_pg;
   PROBLIST_PTR p;

   p_pg = first_probgam_node;
   while (p_pg->p_next)                 /* sets no. of components with        */
      p_pg = p_pg->p_next;              /* proband gametes                    */
   NScCmp = p_pg->component;
   NScCmpAlloc = NScCmp;

   NScSet = malloc (NScCmp * sizeof (int)); /* allocs for components         */
   alloc_chk (WHERE, NScSet);
   ScSetNo = malloc (NScCmp * sizeof (int *));
   alloc_chk (WHERE, ScSetNo);
   NPrGam = malloc (NScCmp * sizeof (int *));
   alloc_chk (WHERE, NPrGam);
   ProbNam = malloc (NScCmp * sizeof (char ***));
   alloc_chk (WHERE, ProbNam);
   ProbIx = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, ProbIx);
   ProbGam = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, ProbGam);

   p_pg = first_probgam_node;
   for (c = 0; c < NScCmp; ++c)         /* sets no. of scoresets for ea comp  */
   {
      NScSet[c] = 0;
      while (p_pg && p_pg->component <= c + 1)
      {
         ++NScSet[c];
         p_pg = p_pg->p_next;
      }
      if (!NScSet[c])                   /* gets no. of scoresets for ea comp  */
      {
         ScSetNo[c] = NULL;
         NPrGam[c] = NULL;              /* none: sets this component to empty */
         ProbNam[c] = NULL;
         ProbIx[c] = NULL;
         ProbGam[c] = NULL;
      }
      else
      {
         ScSetNo[c] = malloc (NScSet[c] * sizeof (int)); /* allocs for sets  */
         alloc_chk (WHERE, ScSetNo[c]);
         NPrGam[c] = malloc (NScSet[c] * sizeof (int));
         alloc_chk (WHERE, NPrGam[c]);
         ProbNam[c] = malloc (NScSet[c] * sizeof (char **));
         alloc_chk (WHERE, ProbNam);
         ProbIx[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, ProbIx[c]);
         ProbGam[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, ProbGam[c]);
      }
   }
   p_pg = first_probgam_node;           /* sets no. of proband gametes for    */
   for (c = 0; c < NScCmp; ++c)         /* ea scoreset                        */
      for (s = 0; s < NScSet[c]; ++s)
      {
         ScSetNo[c][s] = p_pg->scoreset;
         NPrGam[c][s] = p_pg->no_pgams; /* allocs for proband gametes         */
         ProbNam[c][s] = malloc (NPrGam[c][s] * sizeof (char *));
         alloc_chk (WHERE, ProbNam[c][s]);
         ProbIx[c][s] = malloc (NPrGam[c][s] * sizeof (int));
         alloc_chk (WHERE, ProbIx[c][s]);
         ProbGam[c][s] = malloc (NPrGam[c][s] * sizeof (int));
         alloc_chk (WHERE, ProbGam[c][s]);

         p = p_pg->p_pglist;            /* stores names and haplotypes of     */
         for (i = 0; i < p_pg->no_pgams; ++i) /* proband gametes             */
         {
            ProbNam[c][s][i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
            alloc_chk (WHERE, ProbNam[c][s][i]);
            strncpy (ProbNam[c][s][i], p->nam, MAXNAMLEN + 1);
            ProbGam[c][s][i] = p->gamete;
            ++p;
         }
         if (NPrGam[c][s] > MxGam)      /* looks for max no. prob gametes     */
            MxGam = NPrGam[c][s];
         p_pg = p_pg->p_next;
      }
   return;
}

void mak_patlab (void)
{
   int   c, s, i;
   int  *p;
   WNDPAT_PTR p_wp;

   p_wp = first_windpat_node;

   NPtLab = malloc (NScCmp * sizeof (int *)); /* allocs for components       */
   alloc_chk (WHERE, NPtLab);
   PatLab = malloc (NScCmp * sizeof (int **));
   alloc_chk (WHERE, PatLab);

   for (c = 0; c < NScCmp; ++c)
      if (!NScSet[c])                   /* tests for probands for this scset  */
      {
         NPtLab[c] = NULL;
         PatLab[c] = NULL;
      }
      else
      {
         NPtLab[c] = malloc (NScSet[c] * sizeof (int)); /* allocs for scsets */
         alloc_chk (WHERE, NPtLab[c]);
         PatLab[c] = malloc (NScSet[c] * sizeof (int *));
         alloc_chk (WHERE, PatLab);

         for (s = 0; s < NScSet[c]; ++s) /* sets no. of patterns for ea      */
         {                              /* component scoreset                 */
            while (p_wp
                   && ((p_wp->p_next && p_wp->component <= c)
                       || (p_wp->p_next && p_wp->component == c + 1
                           && p_wp->scoreset < ScSetNo[c][s])))
               p_wp = p_wp->p_next;

            if (p_wp && p_wp->component == c + 1
                && p_wp->scoreset == ScSetNo[c][s])
            {
               NPtLab[c][s] = p_wp->no_pats; /* allocs for pattern labels    */
               PatLab[c][s] = malloc (NPtLab[c][s] * sizeof (int));
               alloc_chk (WHERE, PatLab[c][s]);

               p = p_wp->p_pats;        /* stores pattern labels              */
               for (i = 0; i < p_wp->no_pats; ++i, ++p)
                  PatLab[c][s][i] = *p;
            }
            else
            {
               NPtLab[c][s] = 0;        /* sets to no patterns for component  */
               PatLab[c][s] = NULL;     /* and scoreset (or none at all)      */
            }
         }
      }
   return;
}

void mak_affind (void)
{
   int   c, i;
   INDNAM_PTR p_aff;
   char *p;

   if (!(p_aff = first_affnam_node))
   {
      NAffCmp = 0;
      return;
   }
   while (p_aff->p_next)                /* sets no. of components with        */
      p_aff = p_aff->p_next;            /* affected individuals               */

   NAffCmp = p_aff->component;          /* (largest component no.)            */

   NAffInd = malloc (NAffCmp * sizeof (int)); /* allocs for components       */
   alloc_chk (WHERE, NAffInd);
   AffNam = malloc (NAffCmp * sizeof (char **));
   alloc_chk (WHERE, AffNam);

   p_aff = first_affnam_node;
   for (c = 0; c < NAffCmp; ++c)        /* sets no. of affecteds for ea comp  */
   {
      if (p_aff->component == c + 1)
      {
         NAffInd[c] = p_aff->no_nams;   /* allocs for no. affecteds           */
         AffNam[c] = malloc (NAffInd[c] * sizeof (char *));
         alloc_chk (WHERE, AffNam[c]);

         p = p_aff->p_namlist;

         for (i = 0; i < NAffInd[c]; ++i) /* stores names                    */
         {
            AffNam[c][i] = malloc ((MAXNAMLEN + 1) * sizeof (char));
            alloc_chk (WHERE, AffNam[c][i]);
            strncpy (AffNam[c][i], p, MAXNAMLEN + 1);
            p += MAXNAMLEN + 1;
         }
         p_aff = p_aff->p_next;
      }
      else
      {
         NAffInd[c] = 0;                /* none named for this component      */
         AffNam[c] = NULL;
      }
   }
   return;
}

/** 
 * makes SelInt[], SelReal[] and NColData.
 * SelInt[0] ... SelInt[n], where n is the number of integers in the ped file
 * the ith integer is to be stored in data[][i]
 * NColData is the allocation: data[][NColData]
 * SelReal[0] ... SelReal[n], where n is the number of reals in the ped file
 * the ith real is to be stored in quant[][i]
 */
void mak_ped_dat (void)
{
   int   j, tno = 0;
   TRTPED_PTR p_p;

   alloc_sel ();

   SelInt[0] = 4;                       /* (gender)                           */
   NColData = 5;                        /* NColData is 5 or greater           */

   if (first_tlist_node)                /* tests for trait selected           */
      tno = first_tlist_node->no;
   else if ((LM_PVAL || LM_IBDTESTS) && AffTrIn) /* tests for affecteds       */
      tno = AffTrIn;                    /* determined from pedfile trait data */

   if (!tno)
      return;

   if (p_p = first_trtped_node)         /* looks for specified pos for trait  */
   {
      while (p_p->p_next && p_p->no < tno)
         p_p = p_p->p_next;
      if (p_p->no != tno)
         p_p = NULL;
   }

   if (PedDatTyp != QUANTITATIVE)       /* sets up SelInt[]                   */
   {
      if (!p_p)
      {
         SelInt[tno] = NColData;        /* sets record pos from trait no.     */
         ++NColData;

         if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB)
         {
            SelInt[tno + 1] = NColData; /* sets 2nd record pos for data       */
            ++NColData;
         }
      }
      else
      {
         j = p_p->pos[0];               /* sets specified record position     */
         if (SexIn != N)                /* note: SexIn may be UNDEF           */
            --j;
         SelInt[j] = NColData;
         ++NColData;

         if (PedDatTyp == INHERITANCE || PedDatTyp == DISCR_LIAB)
         {
            j = p_p->pos[1];            /* sets 2nd record pos for data       */
            if (SexIn != N)
               --j;
            SelInt[j] = NColData;
            ++NColData;
         }
      }
   }
   else if (PedDatTyp == QUANTITATIVE)  /* sets up SelReal[]                  */
      if (!p_p)
         SelReal[tno - 1] = 1;          /* sets record pos from trait no.     */
      else
         SelReal[p_p->pos[0] - 1] = 1;  /* sets specified record pos          */

   return;
}

/**
  * prints the pedigree record position(s) for trait data
  */

void prnt_ped_dat (void)
{
   int   i, j, ic[2], rcol;

   j = 0;
   ic[0] = 0;
   ic[1] = 0;
   if (SexIn != N)                      /* note: SexIn may be UNDEF           */
      ++j;
   for (i = j; i < NInts; ++i)
      if (SelInt[i])
      {
         if (!ic[0])
            ic[0] = i;
         else if (SelInt[ic[0]] < SelInt[i])
            ic[1] = i;
         else
         {
            ic[1] = ic[0];
            ic[0] = i;
         }
      }

   for (i = 0; i < NReals; ++i)
      if (SelReal[i])
         rcol = i;

   switch (PedDatTyp)
   {
       case INHERITANCE:
          printf (" Inheritance indicators at trait locus are to be read\n"
                  "    from integers %d (pat) and %d (mat)"
                  " in the pedigree record\n\n", ic[0] + 1, ic[1] + 1);
          break;
       case GENOTYPIC:
          printf (" Genotypic data at trait locus are to be read"
                  " from integer %d\n"
                  "    in the pedigree record\n\n", ic[0] + 1);
          break;
       case DISCRETE:
          printf (" Discrete data at trait locus are to be read"
                  " from integer %d\n"
                  "    in the pedigree record\n\n", ic[0] + 1);
          break;
       case DISCR_LIAB:
          printf (" Discrete data and liability classes at trait locus"
                  " are to be read\n"
                  "    from integers %d and %d in the pedigree record\n\n",
                  ic[0] + 1, ic[1] + 1);
          break;
       case QUANTITATIVE:
          printf (" Quantitative data at trait locus are to be read"
                  " from real %d\n" "    in the pedigree record\n\n", rcol + 1);
          break;
   }

   return;
}

/**
 * prints the names of the selected markers, if input.
 */
void prnt_marknam (SIM_PTR p_sel)
{
   int   i, mw, nc, no;

   mw = intWid (1, p_sel->no_mark_map);
   nc = get_colfmt2 (79, mw, LocNamLen, 1);

   if (p_sel->chrom_no)
      fprintf (stdout, " Marker names for chromosome %d:\n", p_sel->chrom_no);
   else
      fprintf (stdout, " Marker names:\n");

   for (i = 0; i < NMarks; ++i)
   {
      if (!p_sel->p_mark)
         no = i + 1;
      else
         no = *(p_sel->p_mark + i);

      if (!(i % nc))
         fprintf (stdout, "\n ");
      fprintf (stdout, " %*d %*s ", mw, no, LocNamLen, MarkNam[i]);
   }
   fprintf (stdout, "\n\n");
   fflush (stdout);

   return;
}

/**
  prints the genotypes for the selected markers for observed individuals.
 */
void prnt_markdat (SIM_PTR p_sel)
{
   int   i, j, n, wa, na, nm, ns, nc = 0;

   if (!NObs)
      return;

   if (p_sel->chrom_no)
      fprintf (stdout, " Marker data for chromosome %d:\n\n", p_sel->chrom_no);
   else
      fprintf (stdout, " Marker data:\n\n");

   na = 0;
   for (i = 1; i < NLoci; ++i)          /* gets no. chars for printng alleles */
      if (NAll[i] > na)
         na = NAll[i];
   wa = intWid (0, na);

   for (i = 0; i < NObs; ++i)
   {
      n = strlen (ObsNam[i]);           /* gets no. chars for printing names  */
      nc = n > nc ? n : nc;
   }

   nm = intWid (0, NMarks);             /* and no. chars for prntng mark no.s */
   ns = 2 + 2 * wa - nm;
   if (ns < 0)
      ns = 0;

   fprintf (stdout, "%*s ", nc, " ");
   for (i = 0; i < NMarks; ++i)
   {
      if (!p_sel->p_mark)
         fprintf (stdout, "%*sM%-*d", ns, " ", nm, i + 1);
      else
         fprintf (stdout, "%*sM%-*d", ns, " ", nm, *(p_sel->p_mark + i));
   }
   fprintf (stdout, "\n\n");

   for (i = 0; i < NObs; ++i)
   {
      fprintf (stdout, " %-*s", nc, ObsNam[i]);
      for (j = 0; j < NMarks; ++j)
      {
         if (!((j - 1) % 14) && j != 1)
            fprintf (stdout, " %*s\n", nc, " ");
         fprintf (stdout, "  %*d %*d", wa, ObsMark[i][j][0], wa,
                  ObsMark[i][j][1]);
      }
      fprintf (stdout, "\n");
   }
   fprintf (stdout, "\n\n");
   fflush (stdout);

   return;
}

void prnt_probgam (void)
{
   int   c, s, i, n, nn, np, nd, nc = 0;

   if (NLocWndw)                        /* prints window info                 */
   {
      NWndw = NLoci + 1 - NLocWndw;
      fprintf (stdout, " Autozygosity will be scored for %d windows of "
               "%d loci\n\n\n", NWndw, NLocWndw);
   }

   for (c = 0; c < NScCmp; ++c)
   {
      for (s = 0; s < NScSet[c]; ++s)
      {
         for (i = 0; i < NPrGam[c][s]; ++i)
         {
            n = strlen (ProbNam[c][s][i]); /* gets no. chars in names for    */
            nc = n > nc ? n : nc;       /* printing                           */
         }
      }
   }
   nn = 80 / (nc + 4);
   nd = intWid (1, MxGam);
   np = 60 / (nd + 2);

   fprintf (stdout, " Proband gametes:\n\n");
   for (c = 0; c < NScCmp; ++c)
   {
      for (s = 0; s < NScSet[c]; ++s)   /* for ea component scoreset          */
      {
         for (i = 0; i < NPrGam[c][s]; ++i) /* prints proband gametes        */
         {
            if (!i)
            {
               fprintf (stdout, " Component %d, scoreset %d:\n\n  ",
                        c + 1, ScSetNo[c][s]);
            }
            else if (!(i % nn))
               fprintf (stdout, "\n  ");
            fprintf (stdout, " %*s", nc, ProbNam[c][s][i]);
            fprintf (stdout, " %d ", ProbGam[c][s][i]);
            if (i == NPrGam[c][s] - 1)
               fprintf (stdout, "\n\n");
         }
         if ((LM_AUTO) && NLocWndw && NPrGam[c][s] && NPtLab[c][s])
            for (i = 0; i < NPtLab[c][s]; ++i) /* prints pattern labels      */
            {
               if (!i)
                  fprintf (stdout, " Window patterns:  ");
               else if (!(i % np))
                  fprintf (stdout, "\n                   ");
               fprintf (stdout, " %*d ", nd, PatLab[c][s][i]);
               if (i == NPtLab[c][s] - 1)
                  fprintf (stdout, "\n\n");
            }
         if (NPrGam[c][s])
            fprintf (stdout, "\n");
      }
   }
   fflush (stdout);

   return;
}

void prnt_affind (void)
{
   if (!NAffCmp && AffTrIn)
   {
      fprintf (stdout, " Affected individuals to be identified by");
      if (TraitTyp == GENOTYPIC)
         fprintf (stdout, " genotypic");
      else if (TraitTyp == DISCRETE)
         fprintf (stdout, " discrete");
      fprintf (stdout, " data\n    for trait %d in the pedigree file\n\n",
               AffTrIn);
   }
   else
      fprintf (stdout, " Affected individuals have been read from"
               " parameter statement\n\n");

   fflush (stdout);

   return;
}

void prnt_trt_typ (void)
{
   if (TraitTyp == INHERITANCE)
      fprintf (stdout, " Data are inheritance indicators at trait locus\n\n");
   if (TraitTyp == GENOTYPIC)
      fprintf (stdout, " Trait data are genotypic\n\n");
   else if (TraitTyp == DISCRETE)
   {
      if (PedDatTyp == DISCR_LIAB)
         fprintf (stdout, " Trait data are discrete with liability classes"
                  " provided\n\n");
      else
         fprintf (stdout, " Trait data are discrete\n\n");
   }
   else if (TraitTyp == QUANTITATIVE)
      fprintf (stdout, " Trait data are quantitative\n\n");
}

void prnt_atz_pars (void)
{
   int   n;

   if (IBDDROP)
   {
      fprintf (stdout, " Number of MC iterations requested:  %d\n\n", NIt);
      return;
   }

   if (LM_MAP || LM_PVAL || LM_IBDTESTS || LM_AUTO
       || LM_LODS || LM_BAYES || LM_MARKERS || LM_SCHNELL || LM_TWOQTL)
   {
      if (SmplSetup == INDEP_LOC)
         fprintf (stdout, " Locus-by-locus sampling to be used for setup\n\n");
      else if (SmplSetup == SEQIMP)
         fprintf (stdout, " Sequential imputation to be used for setup,"
                  " using %d samples \n\n", NSeqImp);
      if (Scan)
      {
         fprintf (stdout, " Sampling is to be by scan");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         else if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every scan\n");
         else
            fprintf (stdout, ", with scores estimated every %d scans\n",
                     ScorIt);
      }
      else
      {
         fprintf (stdout, " Sampling is to be by single step");
         if (ScorIt == UNDEF)
            fprintf (stdout, "\n");
         if (ScorIt == 1)
            fprintf (stdout, ", with scores estimated every step\n");
         else
            fprintf (stdout, ", with scores estimated every %d steps\n",
                     ScorIt);
      }
      if (LM_BAYES && SeqImpIt)
         fprintf (stdout, "         and a restart configuration proposed"
                  " by sequential imputation\n"
                  "         every %d iterations\n\n", SeqImpIt);
      else
         fprintf (stdout, "\n");

      fprintf (stdout, " L-sampler probability set to %.3f\n\n", LSamplProb);

      fprintf (stdout, " The number of MC burn-in iterations is %d\n", NBrnIt);
      if (LM_BAYES)
         if (!InPseudo)
            fprintf (stdout, " and the number of preliminary MCMC iterations is"
                     " %d\n", NPreIt);
         else
            fprintf (stdout, "\n Preliminary MCMC iterations are skipped, as"
                     " pseudo-priors are input\n");
      fprintf (stdout, "\n");
   }
   if (!LM_MAP && NIt != UNDEF)
   {
      fprintf (stdout, " Number of MC iterations requested is %d\n", NIt);
      if (ChkIt != UNDEF)
         fprintf (stdout, " with progress checked every %d iterations\n",
                  ChkIt);
   }

   if (LM_MAP)
   {
      fprintf (stdout, " The number of MC iterations for first MCEM step"
               " is %d\n", NIt);
      fprintf (stdout, " The number of curvature iterations is %d\n", NCurvIt);
      fprintf (stdout, " The number of gradient iterations is %d\n", NGradIt);
      fprintf (stdout, " The number of ascent iterations is %d\n", NAscentIt);
      fprintf (stdout, " The number of iterations to compute LRT statistics"
               " is %d\n", NChiSqIt);
      fprintf (stdout, " Recombination fraction estimates are restricted to"
               " the interval [%.2f, %.2f]\n", RF_bound, .5 - RF_bound);
      fprintf (stdout, " EM convergence for succesive estimates is set to"
               " %.4f\n", EM_bound);
      fprintf (stdout, " SA convergence for succesive estimates is set to"
               " %6.4f\n", SA_bound);
      fprintf (stdout, "\n %d map-estimation %s\n", Redo,
               Redo == 1 ? "run is to be made" : "passes are to be made");
      switch (MapTyp)
      {
          case SEX_SPEC:
             fprintf (stdout, " Gender-specific");
             break;
          case SEX_AVG:
             fprintf (stdout, " Gender-averaged");
             break;
          case BOTH:
             fprintf (stdout, " Both gender-specific and -averaged");
             break;
      }
      fprintf (stdout, " maps will be estimated\n");
   }

   if (LM_PVAL)
   {
      fprintf (stdout, "\n IBD statistics are to be computed\n");
      fprintf (stdout, "    measures:   ");
      fprintf (stdout, "MaxFGL - %s   Spairs - %s   Srobdom - %s\n",
               MaxFGLOn ? "yes" : "no", SpairsOn ? "yes" : "no",
               RobdomOn ? "yes" : "no");
   }
   else if (LM_IBDTESTS)
      if (IBDStats)
      {
         fprintf (stdout, "\n IBD statistics are to be computed\n");
         fprintf (stdout, "    measures:   ");
         fprintf (stdout, "Spairs - %s   Srobdom - %s   Saffect - %s"
                  "   Slambda - %s\n",
                  SpairsOn ? "yes" : "no", RobdomOn ? "yes" : "no",
                  SAffUnaffOn ? "yes" : "no", LambdaOn ? "yes" : "no");
         fprintf (stdout, "    tests:      ");
         fprintf (stdout, "normal - %s   permutation - %s\n",
                  NormalEnable ? "yes" : "no", PermutEnable ? "yes" : "no");
         if (PermutEnable)
            fprintf (stdout, "\n    the number of permutations is %d\n",
                     NPermutIt);
         fprintf (stdout, "\n");
      }
      else if (LikRatStats)
      {
         n = DeltaOn + Lambda_pOn;
         fprintf (stdout,
                  "\n Likelihood-ratio statistics are to be computed\n");
         fprintf (stdout, "    measures:   ");
         fprintf (stdout, "delta - %s   lambda-p - %s\n",
                  DeltaOn ? "yes" : "no", Lambda_pOn ? "yes" : "no");
         fprintf (stdout, "    tests:      ");
         fprintf (stdout, "chi-squared - %s\n", ChiSqEnable ? "yes" : "no");
         if (Lambda_pOn)
         {
            fprintf (stdout, "\n    lambda_a, lambda_u interval:"
                     "  gridpoints - %d  spacing %.4f\n",
                     NumOfSegL, IntervalSegL);
            fprintf (stdout, "    p interval:                 "
                     "  gridpoints - %d  spacing %.4f\n",
                     NumOfSegP, IntervalSegP);
         }
      }
   fprintf (stdout, "\n");

   return;
}

/**
 * prints map showing trial trait positions.
 * 5 print lines are used for ea chromosome segment, showing the trait
 * positions at the start of the chromosome, in ea interval between 
 * markers, and at the end of the chromosome.
 * (a segment consists of as many loci as will fit)
 */
void prnt_vmap (PRNTVMAP_PTR p_vmap, int nmarks, int sex)
{
   PSMAP_PTR p_s;
   double *p_d;
   int  *p_f;
   int   i, j, no_loci;
   /*                                    * first prints the heading           */

   switch (sex)
   {
       case MAL:
          fprintf (stdout, "\n Male T%d positions", p_vmap->no);
          break;
       case FEM:
          fprintf (stdout, "\n Female T%d positions", p_vmap->no);
          break;
       default:
          fprintf (stdout, "\n T%d positions", p_vmap->no);
          break;
   }
   fprintf (stdout, " (cM):\n\n");

   j = 0;                               /* first prints out the beg pos       */
   p_s = p_vmap->p_sintv;
   p_d = p_s->p_locdist;
   p_f = p_s->p_star;

   prntvmap_tr0 ();
   while (j < p_s->no_pos)
   {                                    /* inits lines for nxt chrom segment  */
      no_loci = 1;

      while (j < p_s->no_pos && no_loci < max_line_loc)
      {                                 /* adds next locus and dist           */
         if (j < p_s->no_pos - 1)
            prntvmap_tr (no_loci, *p_f);
         else
            prntmap_mrk (no_loci, p_s->sel_map_mark);
         prntmap_dist (no_loci, *p_d);
         ++j;
         ++p_d;
         ++p_f;
         ++no_loci;
      }
      prntmap_line (no_loci);           /* prints out this map segment        */
      prntmap_tr0b ();
   }

   p_s = p_s->p_next;                   /* now prints trait intv positions    */

   for (i = 1; i < nmarks; ++i)
   {
      j = 0;
      p_d = p_s->p_locdist;
      p_f = p_s->p_star;

      prntmap_mrk0 (p_s->sel_map_mark);
      while (j < p_s->no_pos)
      {                                 /* inits lines for nxt marker intv    */
         no_loci = 1;

         while (j < p_s->no_pos && no_loci < max_line_loc)
         {
            if (j < p_s->no_pos)
               prntvmap_tr (no_loci, *p_f);
            prntmap_dist (no_loci, *p_d);
            ++j;
            ++p_d;
            ++p_f;
            ++no_loci;
         }
         if (j == p_s->no_pos)
         {
            prntmap_mrk (no_loci, p_s->p_next->sel_map_mark);
            prntmap_dist (no_loci, *p_d);
            ++no_loci;
         }
         prntmap_line (no_loci);        /* prints out this map segment        */
         prntmap_tr0b ();
      }
      p_s = p_s->p_next;
   }

   j = 0;                               /* and prints out the end pos         */
   p_d = p_s->p_locdist;
   p_f = p_s->p_star;

   prntmap_mrk0 (p_s->sel_map_mark);
   while (j < p_s->no_pos)
   {                                    /* inits lines for nxt chrom segment  */
      no_loci = 1;

      while (j < p_s->no_pos && no_loci < max_line_loc)
      {                                 /* adds next locus and dist           */
         prntvmap_tr (no_loci, *p_f);
         prntmap_dist (no_loci, *p_d);
         ++j;
         ++p_d;
         ++p_f;
         ++no_loci;
      }
      prntmap_line (no_loci);           /* prints out this map segment        */
      prntmap_tr0b ();
   }
   return;
}

int get_marknam (char *nam, char *p_nm, int imrk)
{
   strncpy (nam, p_nm + (imrk - 1) * (MAXNAMLEN + 1), MAXNAMLEN + 1);

   return strlen (nam);
}
